import myhdl
from myhdl import *

def inc_comb(nextCount, count, n):

    @always(count)
    def logic():
        # do nothing here
        pass

    nextCount.driven = "wire"

    return logic

inc_comb.verilog_code =\
"""
assign $nextCount = ($count + 1) % $n;
"""

inc_comb.vhdl_code =\
"""
$nextCount <= ($count + 1) mod $n;
"""




def main():
    m = 8
    n = 2 ** m
    count = Signal(intbv(0)[m:])
    nextCount = Signal(intbv(0)[m:])
    toVerilog(inc_comb, nextCount, count, n)
    toVHDL(inc_comb, nextCount, count, n)

        
if __name__ == '__main__':
    main()


            
            

    

    
        


                

        




import myhdl
from myhdl import *

from arith_utils import log2ceil
from arith_utils import SLOW, MEDIUM, FAST

def PrefixAnd(width, speed, PI, PO):

    n = width
    m = log2ceil(width)

    def fastPrefix():
        PT = Signal(intbv(0))
        @instance
        def logic():
            while 1:
                yield PI, PT
                PT.next[n:] = PI
                for l in range(1, m+1):
                    for k in range(2**(m-l)):
                        for i in range(2**(l-1)):
                            if (k*2**l + i) < n:
                                PT.next[l*n + k*2**l + i] = \
                                    PT[(l-1)*n + k*2**l + i]
                            if (k*2**l + 2**(l-1) + i) < n:
                                PT.next[l*n + k*2**l + 2**(l-1) + i] = \
                                    PT[(l-1)*n + k*2**l + 2**(l-1) + i] & \
                                    PT[(l-1)*n + k*2**l + 2**(l-1) - 1]
                PO.next = PT[(m+1)*n:m*n]
        return logic

    def slowPrefix():
        PT = Signal(intbv(0))
        @instance
        def logic():
            while 1:
                yield PI, PT
                PT.next[0] = PI[0]
                for i in range(1, n):
                    PT.next[i] = PI[i] & PT[i-1]
                PO.next = PT
        return logic

    if speed == SLOW:
        return slowPrefix()
    elif speed == FAST:
        return fastPrefix()
    else:
        raise NotImplementedError
        
    



import myhdl
from myhdl import *
from myhdl.conversion import analyze

def dff(q, d, clk):

    @always(clk.posedge)
    def logic():
        q.next = d

    return logic


from random import randrange

def test_dff():
    
    q, d, clk = [Signal(bool(0)) for i in range(3)]
    
    dff_inst = dff(q, d, clk)

    @always(delay(10))
    def clkgen():
        clk.next = not clk

    @always(clk.negedge)
    def stimulus():
        d.next = randrange(2)

    return dff_inst, clkgen, stimulus

def simulate(timesteps):
    traceSignals.timescale = "1ps"
    tb = traceSignals(test_dff)
    sim = Simulation(tb)
    sim.run(timesteps)
    sim.quit()

simulate(2000)

def convert():
    q, d, clk = [Signal(bool(0)) for i in range(3)]
    toVerilog(dff, q, d, clk)
    analyze(dff, q, d, clk)
 
convert()
    
